package eu.jonahbauer.raytracing.math;

import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

/**
 * A vector-like object that implements the standard mathematical operations
 * @param <T> the type
 */
public interface IVec<T extends IVec<T>> {
    @Contract(pure = true)
    double get(int i);

    @Contract(pure = true)
    @NotNull T plus(@NotNull T other);

    @Contract(pure = true)
    @NotNull T minus(@NotNull T other);

    @Contract(pure = true)
    @NotNull T times(@NotNull T other);

    @Contract(pure = true)
    @NotNull T times(double d);

    @Contract(pure = true)
    default @NotNull T div(double d) {
        return times(1 / d);
    }

    @Contract(pure = true)
    double @NotNull[] toArray();
}
