package eu.jonahbauer.raytracing.render.canvas;

import eu.jonahbauer.raytracing.render.spectrum.SampledSpectrum;
import eu.jonahbauer.raytracing.render.spectrum.SampledWavelengths;
import eu.jonahbauer.raytracing.render.color.ColorRGB;
import eu.jonahbauer.raytracing.render.color.ColorSpace;
import org.jetbrains.annotations.NotNull;

public interface Canvas {
    /**
     * {@return the width of this canvas}
     */
    int getWidth();

    /**
     * {@return the height of this canvas}
     */
    int getHeight();

    /**
     * Adds a sample to this canvas
     * @param x the pixel x coordinate
     * @param y the pixel y coordinate
     * @param n the index of the sample
     * @param spectrum the sampled spectrum
     * @param lambda the sampled wavelengths
     */
    void add(int x, int y, int n, @NotNull SampledSpectrum spectrum, @NotNull SampledWavelengths lambda);

    /**
     * {@return the color at a given pixel}
     * @param x the pixel x coordinate
     * @param y the pixel y coordinate
     * @param cs the color space of the output
     */
    @NotNull ColorRGB getRGB(int x, int y, @NotNull ColorSpace cs);
}
