package eu.jonahbauer.raytracing.render.spectrum;

import eu.jonahbauer.raytracing.render.color.ColorSpace;
import eu.jonahbauer.raytracing.render.color.SigmoidPolynomial;
import eu.jonahbauer.raytracing.render.color.ColorRGB;
import org.jetbrains.annotations.NotNull;

public final class RGBAlbedoSpectrum implements Spectrum {
    private final @NotNull SigmoidPolynomial polynomial;

    public RGBAlbedoSpectrum(@NotNull ColorSpace cs, @NotNull ColorRGB rgb) {
        if (rgb.r() < 0 || rgb.r() > 1 || rgb.g() < 0 || rgb.g() > 1 || rgb.b() < 0 || rgb.b() > 1) {
            throw new IllegalArgumentException();
        }
        this.polynomial = cs.toSpectrum(rgb);
    }

    @Override
    public double max() {
        return polynomial.max();
    }

    @Override
    public double get(double lambda) {
        return polynomial.get(lambda);
    }
}
