package eu.jonahbauer.raytracing.render.spectrum;

import eu.jonahbauer.raytracing.math.Ray;
import eu.jonahbauer.raytracing.math.Vec3;
import eu.jonahbauer.raytracing.render.color.ColorSpace;
import eu.jonahbauer.raytracing.render.color.ColorXYZ;
import eu.jonahbauer.raytracing.render.color.ColorRGB;
import eu.jonahbauer.raytracing.render.texture.Texture;
import eu.jonahbauer.raytracing.scene.HitResult;
import eu.jonahbauer.raytracing.scene.SkyBox;
import org.jetbrains.annotations.NotNull;

public interface Spectrum extends Texture, SkyBox {
    int LAMBDA_MIN = 360;
    int LAMBDA_MAX = 830;

    /**
     * {@return the maximum value of <code>this</code> spectrum over the range of wavelengths}
     */
    double max();

    /**
     * {@return the value of <code>this</code> spectrum at a given wavelength}
     * @param lambda the wavelength in nanometers
     */
    double get(double lambda);

    default @NotNull Spectrum scale(double scale) {
        return new ScaledSpectrum(this, scale);
    }

    default @NotNull SampledSpectrum sample(@NotNull SampledWavelengths lambdas) {
        return new SampledSpectrum(lambdas, this);
    }

    default @NotNull ColorXYZ toXYZ() {
        return new ColorXYZ(
                Util.innerProduct(Spectra.X, this) / ColorXYZ.CIE_Y_INTEGRAL,
                Util.innerProduct(Spectra.Y, this) / ColorXYZ.CIE_Y_INTEGRAL,
                Util.innerProduct(Spectra.Z, this) / ColorXYZ.CIE_Y_INTEGRAL
        );
    }

    default @NotNull ColorRGB toRGB(@NotNull ColorSpace cs) {
        return cs.toRGB(toXYZ());
    }

    /*
     * Texture
     */

    @Override
    default @NotNull Spectrum get(@NotNull HitResult hit) {
        return this;
    }

    @Override
    default @NotNull Spectrum get(double u, double v, @NotNull Vec3 p) {
        return this;
    }

    @Override
    default boolean isUVRequired() {
        return false;
    }

    /*
     * SkyBox
     */

    @Override
    default @NotNull SampledSpectrum getColor(@NotNull Ray ray) {
        return this.sample(ray.lambda());
    }
}
