package eu.jonahbauer.raytracing.scene;

import eu.jonahbauer.raytracing.math.Ray;
import eu.jonahbauer.raytracing.render.spectrum.SampledSpectrum;
import eu.jonahbauer.raytracing.render.spectrum.Spectrum;
import org.jetbrains.annotations.NotNull;

@FunctionalInterface
public interface SkyBox {
    @NotNull SampledSpectrum getColor(@NotNull Ray ray);

    static @NotNull SkyBox gradient(@NotNull Spectrum top, @NotNull Spectrum bottom) {
        return ray -> {
            // altitude from -pi/2 to pi/2
            var alt = Math.copySign(
                    Math.acos(ray.direction().withY(0).unit().dot(ray.direction().unit())),
                    ray.direction().y()
            );

            return SampledSpectrum.lerp(
                    top.sample(ray.lambda()),
                    bottom.sample(ray.lambda()),
                    alt / Math.PI + 0.5
            );
        };
    }
}
